-- AUTHOR - Kaytotes
-- CHANGELOG - Upped Oxygen to 45 - Added Food

local ImpGTA = {};

local VERSION = "0.2a";

-- CONFIGURATION STARTS HERE --
local ON_BY_DEFAULT = true;
local HIDE_RADAR_ON_FOOT = true;
local DISABLE_PLAYER_ABILITIES = true;
local SHOW_BUILD_NUMBER = true;
local TOGGLE_BUTTON = Keys.F12;
local EAT_FOOD_BUTTON = Keys.E;
-- CONFIGURATION ENDS HERE --

-- Fonts
local FONT_CURLY = 1;
local FONT_THIN_BLOCK = 4;
local FONT_MONEY = 7;

-- Activation / Deactivation Vars
local modEnabled;
local activationCooldown = 0; -- Used to limit key press polling rate
local activatedString = "NULL";
local stringFadeDelayAmount = 150;
local stringFadeTimer = 0;
local debugMode = false;

-- Player Vars
local playerMaxHealth = 150;

-- GUI Vars
local storedHealth = 0;
local statsOffset = 0.0;

-- Hunger / Food
local foodPlaces = {{ -1196, -1542, 4 }, { -1515, -951, 9 }, { -1814, -1191, 13 },
					{ -1689, -1093, 12 }, { -1281, -873, 11 }, { -1183, -883, 13 },
					{ -660, -909, 24 }, { -1042, -1397, 5 }, { -856, -1143, 6 },
					{ -165, -1440, 31 }, { 3, -1606, 29 }, { 129, -1462, 29},
					{ 792, -736, 27 }, { 186, -226, 53 }, { -213, -41, 50 },
					{ -374, 262, 84 }, { -46, 228, 107 }, { 243, 164, 105 },
					{ -629, -2267, 5 }, { 271, -965, 29 }, { 2684, 3281, 55 },
					{ 1730, 6409, 35 }, { 1585, 6449, 25 }, { 1087, 6506, 21 },
					{ -1045, 5328, 44 }, { -2507, 3614, 13 }, { -3417, 967, 8 },
					{ -3047, 615, 7 }, { -1534, 424, 35 }, { 652, 2728, 41 }, { 1475, 2725, 37 }};
local uiBlips = {};
local hungerStatus = { 0, 0, 0 }; -- Each Chars Hunger
local foodHint = "";
local showFoodHint = false;
local activeCharacter;
local foodFrameDelay = 60;

-- Check if Player is in a vehicle
function ImpGTA.IsPlayerDriving(player)
	local vehicle = PED.GET_VEHICLE_PED_IS_USING(player);
	if( VEHICLE.GET_PED_IN_VEHICLE_SEAT(vehicle, -1) == player) then
		return true;
	else
		return false;
	end
end

-- Check if Player is on phone
function ImpGTA.IsPlayerOnPhone(player)
	local onPhone = PED.IS_PED_RUNNING_MOBILE_PHONE_TASK(player);
	if( onPhone ) then
		return true;
	else
		return false;
	end
end

-- Mod Simulation Logic
function ImpGTA.mod_tick()
	-- Current Player Handle
	playerPed = PLAYER.PLAYER_PED_ID();

	-- Which Character is playing?
	if( PED.IS_PED_MODEL(playerPed, GAMEPLAY.GET_HASH_KEY("player_zero")))then
		activeCharacter = 0;
	elseif( PED.IS_PED_MODEL(playerPed, GAMEPLAY.GET_HASH_KEY("player_one")))then
		activeCharacter = 1;
	elseif( PED.IS_PED_MODEL(playerPed, GAMEPLAY.GET_HASH_KEY("player_two")))then
		activeCharacter = 2;
	end

	-- Make sure Blips stay hidden
	PLAYER.SET_POLICE_RADAR_BLIPS( false );
	
	if( HIDE_RADAR_ON_FOOT ) then
		-- Toggle Radar based on Driving Status
		if( ImpGTA.IsPlayerOnPhone(playerPed) or ImpGTA.IsPlayerDriving(playerPed))then--ImpGTA.IsPlayerDriving(playerPed))then
			UI.DISPLAY_RADAR(true);
			statsOffset = 30.0;
		else
			UI.DISPLAY_RADAR(false);
			statsOffset = 1.5;
		end
	else
		statsOffset = 30.0;
	end

	-- Increase Damage Output
	PLAYER.SET_PLAYER_WEAPON_DAMAGE_MODIFIER(PLAYER.PLAYER_ID(), 1.9 );

	-- Adjust Oxygen Amount
	PED.SET_PED_MAX_TIME_UNDERWATER(playerPed, 45.0);

	-- Make Player Die Easier
	if( ENTITY.GET_ENTITY_HEALTH( playerPed ) > playerMaxHealth ) then
		ENTITY.SET_ENTITY_HEALTH( playerPed, playerMaxHealth );
	end

	-- Remove Abilities
	if( DISABLE_PLAYER_ABILITIES ) then
		if( PLAYER.IS_SPECIAL_ABILITY_ACTIVE(PLAYER.GET_PLAYER_INDEX())) then
			PLAYER.SPECIAL_ABILITY_DEACTIVATE(PLAYER.GET_PLAYER_INDEX());
		end
	end

	if( foodFrameDelay < 0)then
		if( hungerStatus[activeCharacter + 1] > 0 )then
			hungerStatus[activeCharacter + 1] = hungerStatus[activeCharacter + 1] - 1;
		end
		foodFrameDelay = 3500;
	else
		foodFrameDelay = foodFrameDelay - 1;
	end

	-- Tweak Accuracy based on Hunger
	if( hungerStatus[activeCharacter + 1] >= 26)then
		PED.SET_PED_ACCURACY(playerPed, 100);
	elseif( hungerStatus[activeCharacter + 1] < 25 and hungerStatus[activeCharacter +1] > 0)then
		PED.SET_PED_ACCURACY(playerPed, 50);
	elseif( hungerStatus[activeCharacter +1] == 0 )then
		PED.SET_PED_ACCURACY(playerPed, 1);
	end

	-- Find Nearest Food Place
	local currentPos = ENTITY.GET_ENTITY_COORDS(playerPed, nil);
	for i, loc in pairs(foodPlaces) do
		if( GAMEPLAY.GET_DISTANCE_BETWEEN_COORDS(currentPos.x, currentPos.y, currentPos.z, loc[1], loc[2], currentPos.z, false) < 6 )then
			if( hungerStatus[activeCharacter + 1] < 100 )then
				ImpGTA.Write("Press 'E' to Eat Food", 0.40, 0.45, 0.7, 255, 255, 255, FONT_CURLY);

				-- Check for Input
				if( hungerStatus[activeCharacter + 1] < 100 and get_key_pressed( EAT_FOOD_BUTTON ))then

					local cashStat = "SP"..activeCharacter.."_TOTAL_CASH";
					local statHash = GAMEPLAY.GET_HASH_KEY(cashStat);
					local bool, currentCash = STATS.STAT_GET_INT(statHash, 0, -1);

					-- Remove Cash
					UI.DISPLAY_CASH(true);
					STATS.STAT_SET_INT(statHash, currentCash - 5, true);

					-- Eat Food
					hungerStatus[activeCharacter + 1] = 100;
					ENTITY.SET_ENTITY_HEALTH( playerPed, playerMaxHealth );

					-- TODO
					-- play animation
				end
			end
			break;
		end
	end

	ImpGTA.DrawGUI(playerPed);
end

function ImpGTA.DrawGUI(player)

	-- Write Build Info
	if( SHOW_BUILD_NUMBER ) then
		if(CUTSCENE.IS_CUTSCENE_PLAYING() == false )then
			ImpGTA.Write("ImpGTA: " .. VERSION, 0.45, 0.0, 0.5, 255, 255, 255, FONT_CURLY);
		end
	end

	-- Only Draw if no Cutscene playing
	if(CUTSCENE.IS_CUTSCENE_PLAYING() == false )then

		-- Draw Oxygen Amount
		if(PED.IS_PED_SWIMMING_UNDER_WATER(player) )then
			local oxygen = math.floor(PLAYER.GET_PLAYER_UNDERWATER_TIME_REMAINING(PLAYER.PLAYER_ID()));
			if( oxygen <= 0 ) then
				oxygen = 0;
			end

			if( oxygen >= 21 )then
				ImpGTA.Write("Oxygen: " .. oxygen, 0.005 * statsOffset, 0.005 * 170.0 , 0.65, 255, 255, 255, FONT_CURLY);
			elseif( oxygen <= 20 and oxygen > 8 ) then
				ImpGTA.Write("Oxygen: " .. oxygen, 0.005 * statsOffset, 0.005 * 170.0 , 0.65, 255, 255, 0, FONT_CURLY);
			elseif( oxygen <= 8 ) then
				ImpGTA.Write("Oxygen: " .. oxygen, 0.005 * statsOffset, 0.005 * 170.0 , 0.65, 255, 0, 0, FONT_CURLY);
			end
		end

		-- Draw Health Display
		local currentHealth = ENTITY.GET_ENTITY_HEALTH(player) - 100;
		if( currentHealth <= 0 ) then
			currentHealth = 0;
		end
		currentHealth = currentHealth * 2; -- Fixes Health Display from Tweaking Player Health
		
		if( currentHealth >= 71 ) then
			ImpGTA.Write("Health: " .. currentHealth, 0.005 * statsOffset, 0.005 * 177.0 , 0.65, 255, 255, 255, FONT_CURLY);
		elseif( currentHealth <= 70  and currentHealth > 30 ) then
			ImpGTA.Write("Health: " .. currentHealth, 0.005 * statsOffset, 0.005 * 177.0, 0.65, 255, 255, 0, FONT_CURLY);
		elseif( currentHealth <= 30) then
			ImpGTA.Write("Health: " .. currentHealth, 0.005 * statsOffset, 0.005 * 177.0, 0.65, 255, 0, 0, FONT_CURLY);
		end

		-- Draw Armour Display
		local currentArmour = PED.GET_PED_ARMOUR(player);
		if( currentArmour <= 0 ) then
			currentArmour = 0;
		end

		if( currentArmour >= 61 ) then
			ImpGTA.Write("Armour: " .. currentArmour, 0.005 * (statsOffset) , 0.005 * 184.0 , 0.65, 255, 255, 255, FONT_CURLY);
		elseif( currentArmour <= 60 and currentArmour > 30 ) then
			ImpGTA.Write("Armour: " .. currentArmour, 0.005 * (statsOffset) , 0.005 * 184.0, 0.65, 255, 255, 0, FONT_CURLY);
		elseif( currentArmour <= 30 ) then
			ImpGTA.Write("Armour: " .. currentArmour, 0.005 * (statsOffset) , 0.005 * 184.0, 0.65, 255, 0, 0, FONT_CURLY);
		end

		-- Draw Food Amount
		if( hungerStatus[activeCharacter + 1] >= 26)then
			ImpGTA.Write("Hunger: " .. hungerStatus[activeCharacter + 1], 0.005 * statsOffset, 0.005 * 191.0 , 0.65, 255, 255, 255, FONT_CURLY);
		elseif( hungerStatus[activeCharacter + 1] <= 25 and hungerStatus[activeCharacter +1] > 0)then
			ImpGTA.Write("Hunger: " .. hungerStatus[activeCharacter + 1], 0.005 * statsOffset, 0.005 * 191.0 , 0.65, 255, 255, 0, FONT_CURLY);
		elseif( hungerStatus[activeCharacter +1] == 0 )then
			ImpGTA.Write("Hunger: " .. hungerStatus[activeCharacter + 1], 0.005 * statsOffset, 0.005 * 191.0 , 0.65, 255, 0, 0, FONT_CURLY);
		end

		-- Draw Debug Stuff (Coords)
		if(debugMode)then
			local currentPos = ENTITY.GET_ENTITY_COORDS(player, nil);		
			ImpGTA.Write("Current Pos: " .. math.floor(currentPos.x) .. " : " .. math.floor(currentPos.y) .. " : " .. math.floor(currentPos.z), 0.45, 0.8, 0.5, 255, 255, 255, FONT_THIN_BLOCK);
		end
	end
end

local function BuildFoodBlips()
	for i, loc in pairs(foodPlaces) do
		uiBlips[i] = UI.ADD_BLIP_FOR_COORD(loc[1], loc[2], loc[3]);
		UI.SET_BLIP_SPRITE(uiBlips[i], 1);
		UI.SET_BLIP_SCALE(uiBlips[i], 0.6);
		UI.SET_BLIP_AS_SHORT_RANGE(uiBlips[i], true);
	end
end

local function DestroyFoodBlips()
	for i, loc in pairs(foodPlaces) do
		UI.REMOVE_BLIP(uiBlips[i]);
	end
end

function ImpGTA.Enable()
	activatedString = "";
	stringFadeTimer = stringFadeDelayAmount;
	activationCooldown = 10; -- Ignore input for 10 Frames

	modEnabled = true;

	-- Remove Police Blips
	PLAYER.SET_POLICE_RADAR_BLIPS( false );

	-- Init Food
	hungerStatus[1] = math.random(70, 100);
	hungerStatus[2] = math.random(70, 100);
	hungerStatus[3] = math.random(70, 100);
	BuildFoodBlips();
end

-- Revert Settings
function ImpGTA.Disable()
	activatedString = "ImpGTA: Disabled";
	stringFadeTimer = stringFadeDelayAmount;
	activationCooldown = 10; -- Ignore input for 10 Frames

	ImpGTA.unload(); -- Clean Up
	modEnabled = false;

	-- Reset All Things
	PLAYER.SET_POLICE_RADAR_BLIPS( true );
	UI.DISPLAY_RADAR( true );
	ENTITY.SET_ENTITY_HEALTH( PLAYER.PLAYER_ID(), 200 );
	PLAYER.SET_PLAYER_WEAPON_DAMAGE_MODIFIER(PLAYER.PLAYER_ID(), 1.0 );

	DestroyFoodBlips();
	PED.SET_PED_ACCURACY(PLAYER.PLAYER_PED_ID(), 100);
end

-- Runs Every Frame
function ImpGTA.tick()

	-- Draw Activation / Deactivation String
	if( stringFadeTimer > 0 )then
		ImpGTA.Write(activatedString, 0.40, 0.45, 0.8, 255, 255, 255, FONT_CURLY);
	end

	activationCooldown = activationCooldown - 1;
	stringFadeTimer = stringFadeTimer - 1;

	-- Turn on Mod
	if( get_key_pressed( TOGGLE_BUTTON ) and modEnabled == false and activationCooldown < 1 ) then
		ImpGTA.Enable();
	end

	-- Turn off Mod
	if( get_key_pressed( TOGGLE_BUTTON ) and modEnabled == true and activationCooldown < 1 ) then
		ImpGTA.Disable();
	end

	if( modEnabled ) then
		ImpGTA.mod_tick();
	end
end

-- Initialise
function ImpGTA.init()
	modEnabled = false;
	if(ON_BY_DEFAULT == true) then
		ImpGTA.Enable();
	end
end

-- Clean up
function ImpGTA.unload()

end

-- Draws Text to the Screen
function ImpGTA.Write( text, x, y, scale, r, g, b, font)
	UI.SET_TEXT_FONT( font );
	UI.SET_TEXT_SCALE( scale, scale );
	UI.SET_TEXT_COLOUR( r, g, b, 255 );
	UI.SET_TEXT_WRAP( 0.0, 1.0 );
	UI.SET_TEXT_CENTRE( false );
	UI.SET_TEXT_DROPSHADOW( 2, 2, 0, 0, 0 );
	UI.SET_TEXT_EDGE( 1, 0, 0, 0, 205 );
	UI._SET_TEXT_ENTRY( "STRING" );
	UI._ADD_TEXT_COMPONENT_STRING( text );
	UI._DRAW_TEXT( x, y );
end

return ImpGTA;